@@ -15,7 +15,6 @@ import platform
 import json
 import warnings
 from datetime import datetime, timedelta
-from distutils.errors import DistutilsFileError
 
 import luigi
 
@@ -49,19 +48,45 @@ class QueenbeeTask(luigi.Task):
         * supports piping
     """
 
-    def _get_log_folder(self, folder):
+    def _get_log_folder(self, folder) -> str:
         """A hack to find the log file!"""
         folder = pathlib.Path(folder)
-        log_file = pathlib.Path(folder, '__logs__', 'logs.cfg')
+        log_file = folder.joinpath('__logs__', 'logs.cfg')
         if log_file.exists():
             return log_file.as_posix()
         else:
             return self._get_log_folder(folder.parent)
 
+    def status_file(self) -> pathlib.Path:
+        folder = pathlib.Path(self._get_log_folder(self.initiation_folder)).parent
+        # the empty status file should be created by queenbee-luigi
+        status_file = folder.joinpath('status.json')
+        if not status_file.is_file():
+            status_file.write_text(
+                json.dumps({
+                    'meta': {
+                        'progress': {
+                            'completed': 0,
+                            'running': 0,
+                            'total': 0
+                        }
+                    }
+                })
+            )
+        return status_file
+
     @property
-    def log_config(self):
+    def log_config(self) -> str:
         return self._get_log_folder(self.initiation_folder)
 
+    @property
+    def task_image(self):
+        raise NotImplementedError('`task_image` should be overwritten by each task.')
+
+    @property
+    def image_workdir(self):
+        raise NotImplementedError('`image_workdir` should be overwritten by each task.')
+
     def get_interface_logger(self):
         logger = logging.getLogger('luigi-interface')
         if not logger.handlers:
@@ -210,6 +235,10 @@ class QueenbeeTask(luigi.Task):
             return False
         return self._input_params['__debug__']
 
+    @property
+    def _run_inside_container(self):
+        return self._input_params.get('__run_inside_container__', False)
+
     def _get_dst_folder(self, command):
         debug_folder = self._is_debug_mode
         dst_dir = tempfile.TemporaryDirectory()
@@ -253,6 +282,22 @@ class QueenbeeTask(luigi.Task):
         self._copy_input_artifacts(dst)
         logger.info(f'Started running {self.__class__.__name__}...')
         qb_logger.info(f'Started running {self.__class__.__name__}...')
+        self._update_status(logger=qb_logger, started=True)
+
+        # TODO: offer an option for podman
+        if self._run_inside_container:
+            dst_path = pathlib.Path(dst)
+            if hasattr(self, 'task_image'):
+                command = 'docker run -it --rm ' \
+                    f'--volume {dst_path.as_posix()}:/home/temp/{dst_path.name} ' \
+                    f'-w /home/temp/{dst_path.name} ' \
+                    f'{self.task_image} {command}'
+            else:
+                message = 'This recipe is generated by an older version of ' \
+                    'queenbee-luigi and cannot be executed in a container.'
+                logger.info(message)
+                qb_logger.info(message)
+
         p = subprocess.Popen(
             command, stdout=subprocess.PIPE,
             stderr=subprocess.STDOUT, shell=True, env=os.environ
@@ -270,6 +315,7 @@ class QueenbeeTask(luigi.Task):
                 'Run the command with --debug key to debug the workflow.\n\n'
             qb_logger.error(err_msg)
             logger.error(err_msg)
+            self._update_status(logger=qb_logger, started=False)
             raise ValueError(err_msg)
 
         # copy the results file back
@@ -290,10 +336,11 @@ class QueenbeeTask(luigi.Task):
         duration -= timedelta(microseconds=duration.microseconds)  # remove microseconds
         logger.info(f'...finished running {self.__class__.__name__} in {duration}')
         qb_logger.info(f'...finished running {self.__class__.__name__} in {duration}')
+        self._update_status(logger=qb_logger, started=False)
 
     @staticmethod
     def load_input_param(input_param):
-        """A static class kept here for backwards compatability.
+        """A static class kept here for backwards compatibility.
 
         Use the `load_input_param` function directly instead.
         """
@@ -304,6 +351,34 @@ class QueenbeeTask(luigi.Task):
         )
         return load_input_param(input_param)
 
+    def _update_status(self, logger, started=True):
+        """Update run status."""
+        if not hasattr(self, '_status_lock'):
+            warnings.warn(
+                'This recipe is generated by an older version of queenbee luigi. '
+                'Update the recipe to get the run status.'
+            )
+            return
+        with self._status_lock:
+            # open JSON file
+            inf = self.status_file()
+            # update values
+            data = json.loads(inf.read_text())
+            progress = data['meta']['progress']
+            if started:
+                progress['running'] += 1
+                progress['total'] += 1
+            else:
+                # task finished
+                progress['running'] -= 1
+                progress['completed'] += 1
+            # write the json file
+            logger.info(
+                f'[{progress["completed"]}/{progress["total"]}] completed. '
+                f'{progress["running"]} running.'
+            )
+            inf.write_text(json.dumps(data))
+
 
 def load_input_param(input_param):
     """This function tries to import the values from a file as a Task input
